#include <iostream>
#include <fstream>
#include <vector>
#include "bmp.hpp"

char* bmp_m = "BM";

bool writeBmp(std::vector<std::vector<px_24> >& picture, const char* name, int width, int height)
{
    bmp_h header;
    dib_h infos;
    int size = width * height;
    // First Header
    uint32_t fileSize = 2 + sizeof(bmp_h) + sizeof(dib_h) + (sizeof(px_24) * size);
    header.fileSize  = fileSize;
    header.creator1  = 0;
    header.creator2  = 0;
    header.bmpOffset = 0x36;
    // Second Heaer
    infos.headerSize    = sizeof(dib_h);
    infos.width         = width;
    infos.height        = height;
    infos.nplanes       = 1;
    infos.bitspp        = 24;
    infos.compress_type = 0;
    infos.bmp_bytesz    = 0;
    infos.hres          = 0;
    infos.vres          = 0;
    infos.ncolors       = 0;
    infos.nimpcolors    = 0;
    // Init file stream
    std::ifstream::pos_type sz;
    std::ofstream output(name, std::ios::out | std::ios::binary);
    // Write headers
    output.write(bmp_m, 2);
    output.write((char*) &header, sizeof(header));
    output.write((char*) &infos, sizeof(infos));
    int paddedLine = (width*3) + ((width*3)%4);
    // Write bmp
    for (int i = height-1; i >= 0; i--)
    {
        std::vector<px_24> input = picture.at(i);
        uint8_t line[paddedLine];
        for (int j = 0; j < width; j++)
        {
            // RGB is in little endian...
            line[(j*3)]   = input.at(j).B;
            line[(j*3)+1] = input.at(j).G;
            line[(j*3)+2] = input.at(j).R;
        }
        // Padding for non-4 multiples
        for (int j = width*3; j < paddedLine; j++)
        {
            line[j] = 0x00;
        }
        output.write((char*) line, paddedLine);
    }
}
